'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.fluxEnhancer = fluxEnhancer;

var _lodash = require('lodash');

var _lodash2 = _interopRequireDefault(_lodash);

var _objectMerge = require('object-merge');

var _objectMerge2 = _interopRequireDefault(_objectMerge);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

function initializeStoreMap(stateStoreMap) {
  var invertedMap = _lodash2.default.transform(stateStoreMap, function (map, store, prop) {
    return map.set(store, prop);
  }, new Map());

  var waitingSet = new Set();
  var storeMap = new Map();
  _lodash2.default.forEach(stateStoreMap, function (store) {
    return waitingSet.add(store);
  });

  var assignStore = function assignStore(store) {
    if (storeMap.has(store)) return;
    if (store === undefined || !waitingSet.has(store)) {
      throw new Error('Undefined store, check for circular dependencies in storeDependencies');
    }

    waitingSet.delete(store);
    if (Array.isArray(store.storeDependencies)) {
      store.storeDependencies.forEach(function (dependency) {
        return assignStore(dependency);
      });
    }
    storeMap.set(store, invertedMap.get(store));
  };

  while (waitingSet.size > 0) {
    var store = waitingSet.entries().next().value[0];
    assignStore(store);
  }

  return storeMap;
}

function fluxEnhancer(stateStoreMap) {
  return function (storeCreator) {
    return function (reducer, initialState) {

      var storeMap = initializeStoreMap(stateStoreMap);
      var partiallyReducedState = null;
      var reduceFromStores = function reduceFromStores() {
        var state = arguments.length <= 0 || arguments[0] === undefined ? {} : arguments[0];
        var action = arguments[1];

        var newState = {};
        partiallyReducedState = _lodash2.default.assign({}, state);

        var _iteratorNormalCompletion = true;
        var _didIteratorError = false;
        var _iteratorError = undefined;

        try {
          for (var _iterator = storeMap[Symbol.iterator](), _step; !(_iteratorNormalCompletion = (_step = _iterator.next()).done); _iteratorNormalCompletion = true) {
            var entry = _step.value;

            var key = entry[1];
            var _store = entry[0];
            if (!_store.reduce) throw new Error(_store.constructor.name + ' must provide a reduce function');

            newState[key] = _store.reduce.bind(_store)(state[key], action);
            partiallyReducedState[key] = newState[key];
          }
        } catch (err) {
          _didIteratorError = true;
          _iteratorError = err;
        } finally {
          try {
            if (!_iteratorNormalCompletion && _iterator.return) {
              _iterator.return();
            }
          } finally {
            if (_didIteratorError) {
              throw _iteratorError;
            }
          }
        }

        partiallyReducedState = null;
        return newState;
      };

      var augmentedReducer = function augmentedReducer(state, action) {
        var storeResult = reduceFromStores(state, action);
        var reducerResult = reducer(state, action);
        return (0, _objectMerge2.default)(reducerResult, storeResult);
      };

      var store = storeCreator(augmentedReducer, initialState);
      var storeDotGetState = store.getState;
      store.getState = function () {
        return partiallyReducedState || storeDotGetState();
      };

      return store;
    };
  };
}